# +---------------------------------------------------------+
# | Copyright (c) 2004 Anthony D'Agostino                   |
# | http://ourworld.compuserve.com/homepages/scorpius       |
# | scorpius@compuserve.com                                 |
# | January 24, 2004                                        |
# | WinOSi Export for Blender 2.36                          |
# +---------------------------------------------------------+

import Blender, mod_meshtools
import math, os
from time import *

# ================================
# === Write WinOSI Mesh Format ===
# ================================
def write_mesh(file, object, material_name):
	mesh = Blender.NMesh.GetRaw(object.data.name)

	# Begin Mesh
	object_name = object.name
	file.write("PolyMesh " + '"'+object_name+'"' + " {\n")
	#file.write("\tdoublesided rounded Material " + '"'+material_name+'"' + "\n")
	file.write("\tsinglesided flat Material " + '"'+material_name+'"' + "\n")

	# Write Verts
	file.write("\tPoints ")
	file.write("%i\n" % len(mesh.verts))
	for vertex in mesh.verts:
		x, y, z = mod_meshtools.apply_transform(vertex.co, object.matrix)
		file.write("\t%f %f %f\n" % (x,y,z))

	# Write Faces
	file.write("\tFaces ")
	file.write("%i\n" % len(mesh.faces))
	for face in mesh.faces:
		file.write("\t%d" % len(face.v))
		face.v.reverse()
		for vertex in face.v:
			file.write(" %d" % (vertex.index))
		file.write("\n")

	# End Mesh
	file.write("}\n\n")

# ====================
# === Write Meshes ===
# ====================
def write_meshes(file):
	file.write("!====================!\n")
	file.write("!=== Mesh Objects ===!\n")
	file.write("!====================!\n")
	objects = Blender.Object.Get()
	objects.sort(lambda a,b: cmp(a.name, b.name))
	object_names = map(getattr, objects, ["name"]*len(objects))
	for object in objects:
		mesh = Blender.NMesh.GetRaw(object.data.name)
		if mesh:
			if mesh.materials and mesh.materials[0]:
				material_name = mesh.materials[0].name
			write_mesh(file, object, material_name)

# ========================
# === Write VMD Header ===
# ========================
def write_header(file):
	timestamp = strftime("%A, %B %d, %Y (%I:%M:%S PM)", localtime(time()))
	blender_file = Blender.Get('filename')
	blender_version = Blender.Get('version')
	blender_version_str = `blender_version`[0] + '.' + `blender_version`[1:]
	header =  "!==================================================!\n"
	header += "!=== WinOSI Export Script by Anthony D'Agostino ===!\n"
	header += "!==================================================!\n\n"
	header += "VMODEL VERSION 0.63\n\n"
	header += "OBJECT: " + blender_file + "\n"
	header += "AUTHOR: Blender " + blender_version_str + "\n"
	header += "DATE  : " + timestamp + "\n\n"
	header += "Settings {\n"
	header += "\tHitBuffers = 5000\n"
	header += "\tDiffDepth  = 3\n"
	header += "\tSpecDepth  = 0\n"
	header += "\tGlobalIOR  = 1.0\n"
	header += "\tBackground = 0.0 0.0 0.0\n"
	header += "\tAmbient    = 0.3 0.3 0.3\n"
	header += "}\n\n"
	file.write(header)

# =======================
# === Write Materials ===
# =======================
def write_materials(file):
	file.write("!=================!\n")
	file.write("!=== Materials ===!\n")
	file.write("!=================!\n")
	materials = Blender.Material.Get()
	material_names = map(getattr, materials, ["name"]*len(materials))
	for material in materials:
		if not material: continue
		material_name = material.name
		file.write("Material " + '"'+material_name+'"' + " {\n")
		file.write("\tReflection RGB %f %f %f\n" % (material.R, material.G, material.B))
		if material.emit: file.write("\tEmission %f\n" % 1.0) #(material.emit))
		file.write("}\n\n")
	file.write("")

# =====================
# === Write Cameras ===
# =====================
def write_cameras(file):
	objects = Blender.Object.Get()
	for object in objects:
		try:
			camera = Blender.Camera.Get(object.data.name)
			fovh = 2 * math.atan(16/camera.lens) #in radians
			fr_vec = tuple(object.loc)
			look_vec = (-object.matrix[2][0], -object.matrix[2][1], -object.matrix[2][2])
			to_vec = (fr_vec[0]+look_vec[0], fr_vec[1]+look_vec[1], fr_vec[2]+look_vec[2])
			up_vec = (object.matrix[1][0], object.matrix[1][1], object.matrix[1][2])
			file.write("View {\n")
			file.write("\tCamera = %f %f %f\n" % fr_vec)
			file.write("\tTarget = %f %f %f\n" % to_vec)
			file.write("\tUp     = %f %f %f\n" % up_vec)
			file.write("\tFovAng = %f\n" % fovh)
			file.write("}\n\n")
		except:
			pass # it's not a camera
			
# ============
# === main ===
# ============
def main():
	vmd_directory = "C:/WinOSi/"
	vmd_filename = Blender.Get('filename')
	vmd_filename = os.path.basename(vmd_filename)
	vmd_filename, ext = os.path.splitext(vmd_filename)
	vmd_filename = "~" + vmd_filename + ".vmd"
	vmd_filename = os.path.join(vmd_directory, vmd_filename)
	vmd_filename = os.path.normcase(vmd_filename)
	file = open(vmd_filename, "wb")
	write_header(file)
	write_cameras(file)
	write_materials(file)
	write_meshes(file)
	file.close()
	print "\a\r",
	print "Exported to:", vmd_filename

main()